%
%    Copyright (C) 2008  Colin Caprani - www.colincaprani.com
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

function PEout = Magnel(Mt,Ms,plotflag,vargin)
% Calculates maximum and minimum prestress force and eccentricities given
% the transfer and working moments as well as the section properties. Also
% given are the eccentricity limits for a design value of prestress force,
% Pdes
% Units are:
% P - kN | M - kNm | e - mm | Zb,Zt - mm3 | A - mm2

    [P ec loss_t loss_s f_allow] = get_ps_props();
    [A Zt Zb I yt yb] = get_sec_props();
    pl = loss_s; emaxphys = yb - 30; Pdes = P;
    Mt = Mt*1e6; Ms = Ms*1e6; Pdes = Pdes*1e3;
    f_allow = f_allow(:);
    
    if nargin == 4
        Pdes = vargin*1e3;
    end
    
    ftt = f_allow(1); fct = f_allow(2); % transfer
    ftw = f_allow(3); fcw = f_allow(4); % working

    % Line 1 - Allowable tensile stress at transfer (TOP)
    fmtt = Mt/Zt;                     denom = (ftt-fmtt);
    m(1) = -1/(Zt*denom); c(1) = 1/(A*denom);

    % Line 2 - Allowable comp stress at transfer    (BTM)
    fmtb = Mt/Zb;                     denom = (fct+fmtb);
    m(2) = 1/(Zb*denom); c(2) = 1/(A*denom);

    
    % Line 3 - Allowable comp stress at working     (TOP)
    fmwt = Ms/Zt;                     denom = (fcw-fmwt);
    m(3) = -pl/(Zt*denom); c(3) = pl/(A*denom);
    
    % Line 4 - Allowable tensile stress at working  (BTM)
    fmwb = Ms/Zb;                     denom = (ftw+fmwb);
    if denom == 0
        denom = 1e-5; % an approximation to avoid NaNs
    end
    m(4) = pl/(Zb*denom); c(4) = pl/(A*denom);
    
    [x_int14 y_int14] = intersection(m(1),c(1),m(4),c(4));
    emax = min(x_int14,emaxphys);
    Pmin = 1/(emax*m(4)+c(4));
    [emin Pmax] = intersection(m(3),c(3),m(4),c(4));
    if m(3) > 0 % line 3 may govern if slope positive
        Pmin = min(Pmin,1/(emax*m(3)+c(3)));
        [emin Pmax] = intersection(m(2),c(2),m(3),c(3));
    end
    Pmax = 1/Pmax;
    
    emaxdes = min( (1/Pdes-c(1))/m(1), (1/Pdes-c(2))/m(2));
    emaxdes = min(emaxdes,emaxphys);
    emindes = max( (1/Pdes-c(3))/m(3), (1/Pdes-c(4))/m(4) );
    Pmaxdes = 1/max( m(1)*ec+c(1), m(2)*ec+c(2));
    if m(3) > 0
        Pmindes = 1/min( m(3)*ec+c(3), m(4)*ec+c(4));
    else
        Pmindes = 1/(m(4)*ec+c(4));
    end
     
    Pmax = Pmax*1e-3; Pmin = Pmin*1e-3;
    PEout = [Pmax Pmin emax emin emaxdes emindes];   

    if plotflag == 1
        figure;
        xmin = -1.5*Zb/A; xmax = 1.5*x_int14;
        ymin = -0.5*y_int14; ymax = 1.5*y_int14;
        ediag = [xmin:(xmax-xmin)/100:xmax]; hold on;
        clr = ['r' 'g' 'b' 'k'];
        for i = 1:4
            plot(ediag,ediag*m(i)+c(i),clr(i));
        end
        plot([emaxphys emaxphys],[ymin ymax],'c');
        plot([emax emin],[1/Pmin 1/Pmax],'k.');
        plot([emindes emaxdes],[1/Pdes 1/Pdes],'ro');
        plot([xmin emaxdes],[1/Pdes 1/Pdes],'r--');
        plot([emindes emindes],[ymin 1/Pdes],'r--');
        plot([emaxdes emaxdes],[ymin 1/Pdes],'r--');
        
        plot([ec ec],[1/Pmaxdes 1/Pmindes],'bo');
        plot([xmin ec],[1/Pmaxdes 1/Pmaxdes],'b--');
        plot([xmin ec],[1/Pmindes 1/Pmindes],'b--');
        plot([ec ec],[ymin 1/Pmindes],'b--');
        
        axis([xmin xmax ymin ymax]);
        legend('Line 1: T-T-Top','Line 2: T-C-Btm','Line 3: W-C-Top',...
            'Line 4: W-T-Btm','Line 5: Physical Max e','Location','Northwest');
        s1 = sprintf('For P = %d kN',round(Pdes*1e-3));
        s2 = sprintf('   e max = %d mm',round(emaxdes));
        s3 = sprintf('   e min = %d mm',round(emindes));
        s4 = sprintf('For e = %d mm',round(ec));
        s5 = sprintf('   Pmax = %d kN',round(Pmaxdes*1e-3));
        s6 = sprintf('   Pmin = %d kN',round(Pmindes*1e-3));
        annotation1 = annotation('textbox',[0.75 0.25 0.4 0.14],...
          'FitHeightToText','on','EdgeColor','none','String',{s1,s2,s3,s4,s5,s6});
        title('Magnel Diagram'); xlabel('e (mm)'); ylabel ('1/P (kN^-1)');
    end

end

function [x y] = intersection(m1,c1,m2,c2)
    x = (c2-c1)/(m1-m2); % max eccentricity
    y = x*m2+c2;
end